#include"cputest.h"
#include"env.h"

double testCPU_MFLOPS(int N,unsigned *mem_alloc) {
	double *a,*b,*c;
	double t0,t1,t;
	double MFLOPS;
	int times=1;
	unsigned mbytes = N * N * sizeof(double);
	unsigned i;
	double tt0,tt1,tt;
	a = malloc(mbytes);
	b = malloc(mbytes);
	c = malloc(mbytes);
		/* Kbytes of memory allocated for matrices */
	(*mem_alloc) = (mbytes * 3) >> 10;
	if(a == NULL || b == NULL || c == NULL)
		finish("Memory allocation (%dK) failed",(mbytes*3) >> 10);
	for(i = 0; i < N*N; i ++)
		a[i] = b[i] = c[i] = 1.00;

/* for small matrices, iterate many times */
	if(N <= 128) times = 100;

	tt0 = time(NULL);			
	t0 = Wtime();

	for(i=0;i<times;i++)
		mult_matrices(c,a,b,N);

	t1 = Wtime();
	tt1 = time(NULL);

	t = (t1 - t0)/(double)times;
 	tt = (tt1 - tt0)/(double)times;

	if(t <= 0) 	{
		printf("\nZero multiplication time, sorry\n");
		MFLOPS = 0;
	}
	else    {
		MFLOPS = 2.0/t*(double)N*(double)N*(double)N/1000000.0;
#ifdef DEBUG
		printf("times = %d wtime delta = %g time delta = %g\n",times,t,tt);
#endif
	}
	free(a);
	free(b);
	free(c);
	return MFLOPS;
}

/* CPU test if running alone */
int main(int argc,char *argv[]) {
	unsigned NMIN,NMAX,N,mem_alloc;
	double mflops;
	printf("Running CPU test...\n");
	initTiming();
	test_wtime_latency();

	if(argv[1] == NULL) {
		NMIN = 4;
		NMAX = 512;
	}
	else if(argv[2] == NULL) {
		NMIN = NMAX = atoi(argv[1]);
	}
	else {
		NMIN = atoi(argv[1]);
		NMAX = atoi(argv[2]);
	}
	printf("\nSize\tMemory\tMFLOPS\n");
	for(N = NMIN; N <= NMAX; N *= 2) {
		mflops = testCPU_MFLOPS(N,&mem_alloc);
		printf("%d\t%d\t%.2lf\n",N,mem_alloc,mflops);
	}
	return 0;
	
}
